# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""The Achille objects involved in the descriptor parsing.

The descriptor file tells Achille all it needs to build correctly a site.
This includes the different sections, the indexes to build, the files to
transform or to copy, the XSL stylesheets to use and so on.

Everytime Achille is run, the descriptor file is read to build an internal
representation of its content."""


import os, os.path, sys

from Ft.Lib import Uri
from Ft.Xml import Domlette, XPath, EMPTY_NAMESPACE

from Ft.Xml.Domlette import implementation
from Ft.Xml.Domlette import PrettyPrint

from achille.lib import files

# FIXME
from achille.config.constants import *
from achille.config import site

# [TODO] move these constants and the constant module ones into concerned classes
# [TODO] check command line should print human readable information on the site

SITE_TITLE_XPATH      = '/site/title'
SECTIONS_XPATH        = '/site/sections'
SECTION_WITH_ID_XPATH = '/site/sections/section[attribute::id="%s"]'
XSLT_SHEETS_XPATH     = '/site/xslt-set'
CSS_SHEETS_XPATH      = '/site/css-set/css-sheet'

# Section Nodes
NODE_SECTION_TITLE    = 'title'
NODE_SECTION_ARTICLES = 'articles'
NODE_SECTION_FILES    = 'files'
NODE_SECTION_FILE     = 'file'
NODE_SECTION_INDEX    = 'index'
NODE_SECTION_ID       = 'section-id'

NODE_FEATURE_FILE     = 'feature-file'
NODE_IGNORE_FILE      = 'ignore-file'

# Section Attributes
SECTION_ATTR_ID       = 'id'
SECTION_ATTR_ROOT     = 'root'
SECTION_ATTR_SRC      = 'src'
SECTION_ATTR_TARGET   = 'target'

CSS_ATTR_NAME         = 'name'
CSS_ATTR_TYPE         = 'type'

XSLT_ATTR_TYPE        = 'type'
XSLT_ATTR_ROOT        = 'root'


class ConfigurationError(Exception):
    """Used to stop on an error in the descriptor file"""


class Source(object):
    def __init__(self, aRoot, aPath):
        self.__dict__['root'] = aRoot
        self.__dict__['path'] = aPath

    def __setattr__(self, aName, aValue):
        raise AttributeError('readonly %s' % aName)

    def __str__(self):
        lPath = ''
        if self.path.startswith(os.sep) or os.path.splitdrive(self.path)[0] != '':
            lPath += self.path
        else:
            lPath += os.path.join(self.root, self.path)
        return os.path.normpath(lPath)
    
    def __call__(self):
        return str(self)



class Index(object):
    """An abstraction of a section index.
    
    An index is composed of a type, a list of directories and a list
    of ignored files. All its attributes are readonly."""
    
    def __init__(self, aDescriptor, aRoot, aNode):
        self.__dict__['descriptor'] = aDescriptor
        self.__dict__['root'] = aRoot
        self.__dict__['featuredFile'] = ''
        self.__dict__['ignoredFiles'] = []
        self.__dict__['type'] = aNode.getAttributeNS(
            None, INDEX_ATTR_TYPE).capitalize()
        if self.type == '':
            raise ConfigurationError, 'type attribute is mandatory in index elements'

        self.__dict__['sort'] = aNode.getAttributeNS(
            None, INDEX_ATTR_SORT).capitalize()
        if self.sort == '':
            self.__dict__['sort'] = INDEX_SORT_TITLE
            
        try:
            self.__dict__['max'] = int(aNode.getAttributeNS(None,
                                                            INDEX_ATTR_MAX))
        except ValueError:
            self.__dict__['max'] = 0
            
        if self.__dict__['type'] == '':
            self.__dict__['type'] = INDEX_SORT_TITLE

        # [TODO] check the id sanity
        sectionId = aNode.parentNode.getAttributeNS(None, SECTION_ATTR_ID)
        self.__dict__['sectionIds'] = [sectionId]
        self.__parseNode(aNode)

    def __setattr__(self, aName, aValue):
	if not aName.startswith('_Index__'):
            raise AttributeError('readonly %s' % aName)

    def __parseNode(self, aNode):
        for lNode in aNode.childNodes:
            if lNode.nodeName == NODE_SECTION_ID:
                self.sectionIds.append(lNode.firstChild.nodeValue)

            # [TODO] precedence rules
            elif lNode.nodeName == NODE_INDEX_PRECEDENCE:
                pass

            elif lNode.nodeName == NODE_FEATURE_FILE:
	        lSource = Source(self.root, lNode.firstChild.nodeValue)
                self.__dict__['featuredFile'] = lSource()

            elif lNode.nodeName == NODE_IGNORE_FILE:
                lSource = Source(self.root, lNode.firstChild.nodeValue)
                self.ignoredFiles.append(lSource())
                
    def __getSectionsList(self):
        lDirs = []
        for lSectionId in self.sectionIds:
            lDirs.append(self.descriptor.getSection(lSectionId).source())
        return lDirs
    dirs = property(__getSectionsList)

    def setMax(self, aMax):
        try:
            self.__dict__['max'] = aMax
        except ValueError:
            raise AttributeError('aMax shoud be an int')

    def setType(self, aType):
        try:
            lType = aType.capitalize()
            if aType in INDEX_TYPES:
                self.__dict__['type'] = lType
            else:
                raise AttributeError('Invalid type')
        except ValueError:
            raise AttributeError('aType shoud be a string')
            

    def setSort(self, aSort):
        try:
            lSort = aSort.capitalize()
            if aSort in INDEX_SORTS:
                self.__dict__['sort'] = lSort
            else:
                raise AttributeError('Invalid sort')
        except ValueError:
            raise AttributeError('aSort shoud be a string')


    def toXml(self, aDocument):
        lNode = aDocument.createElementNS(EMPTY_NAMESPACE, 'index')
        lNode.setAttributeNS(EMPTY_NAMESPACE, INDEX_ATTR_TYPE, self.type)
        lNode.setAttributeNS(EMPTY_NAMESPACE, INDEX_ATTR_SORT, self.sort)
        lNode.setAttributeNS(EMPTY_NAMESPACE, INDEX_ATTR_MAX, str(self.max))

        if self.featuredFile != '':
            lFeaturedNode = aDocument.createElementNS(
                EMPTY_NAMESPACE, NODE_FEATURE_FILE)
            lFeaturedNode.appendChild(aDocument.createTextNode(self.featuredFile))
            lNode.appendChild(lFeaturedNode)

        for lId in self.sectionIds:
            lSectionIdNode = aDocument.createElementNS(
                EMPTY_NAMESPACE, NODE_SECTION_ID)
            lSectionIdNode.appendChild(aDocument.createTextNode(lId))
            lNode.appendChild(lSectionIdNode)

        for lFiles in self.ignoredFiles:
            lFilesNode = aDocument.createElementNS(
                EMPTY_NAMESPACE, NODE_IGNORE_FILE)
            lFilesNode.appendChild(aDocument.createTextNode(lFiles))
            lNode.appendChild(lFilesNode)

        return lNode



class Section(object):
    """An abstraction of a site section.

    Each section has source and target directories, and possibly a title and
    an index. 
    """

    increment = 1
    
    def __init__(self, aDescriptor, aRoot, aNode):
        self.descriptor = aDescriptor
        self.index = None
        self.title = u''
        self.copiedFiles = []
        self.transformedFiles = 0
        
        if aNode != None:
            self._parseNode(aRoot, aNode)
        else:
            self.id = 'section' + str(Section.increment)
            self.title = u'New Section ' + str(Section.increment)
            self.source = ''
            self.target = '/'
            Section.increment = Section.increment + 1


    def _parseNode(self, aRoot, aNode):
        self.id = aNode.getAttributeNS(None, SECTION_ATTR_ID)
        if self.id == '':
            raise ConfigurationError, \
                  'id attribute is mandatory in section elements'
        
        self.source = Source(aRoot, aNode.getAttributeNS(None,
                                                         SECTION_ATTR_SRC))
        if self.source == '':
            raise ConfigurationError, \
                  'src attribute is mandatory in section elements'
        
        self.target = aNode.getAttributeNS(None, SECTION_ATTR_TARGET)
        if self.target == '':
            raise ConfigurationError, \
                  'target attribute is mandatory in section elements'
        
        if not self.target.startswith('/'):
            raise ConfigurationError, 'target path must be absolute'
        
        for lNode in aNode.childNodes:
            if lNode.nodeName == NODE_SECTION_TITLE:
                self.title = lNode.firstChild.nodeValue
            
            elif lNode.nodeName == NODE_SECTION_ARTICLES:
                self.transformedFiles = 1
                    
            elif lNode.nodeName == NODE_SECTION_FILES:
                self.copiedFiles.append(lNode.firstChild.nodeValue)
                    
            elif lNode.nodeName == NODE_SECTION_INDEX:
                self.index = Index(self.descriptor, self.source(), lNode)


    def toXml(self, aDocument):
        lNode = aDocument.createElementNS(EMPTY_NAMESPACE, 'section')
        lNode.setAttributeNS(EMPTY_NAMESPACE, 'id', self.id)
        lNode.setAttributeNS(EMPTY_NAMESPACE, 'src', str(self.source))
        lNode.setAttributeNS(EMPTY_NAMESPACE, 'target', self.target)

        lTitleNode = aDocument.createElementNS(EMPTY_NAMESPACE, 'title')
        lTitleNode.appendChild(aDocument.createTextNode(self.title))
        lNode.appendChild(lTitleNode)

        if self.index != None and self.index.type != INDEX_TYPE_NONE:
            lNode.appendChild(self.index.toXml(aDocument))
        
        if self.transformedFiles:
            lArticlesNode = aDocument.createElementNS(EMPTY_NAMESPACE, 'articles')
            lNode.appendChild(lArticlesNode)

        for lFiles in self.copiedFiles:
            lFilesNode = aDocument.createElementNS(EMPTY_NAMESPACE, 'files')
            lFilesNode.appendChild(aDocument.createTextNode(lFiles))
            lNode.appendChild(lFilesNode)

        return lNode



class SectionsSet(list):
    def __init__(self, aDescriptor = None, aNode = None):
        list.__init__(self)
        if aNode != None:
            lRoot = aNode.getAttributeNS(None, SECTION_ATTR_ROOT)
            for lNode in aNode.childNodes:
                if lNode.nodeName == 'section':
                    self.append(Section(aDescriptor, lRoot, lNode))

    def __str__(self):
        lString = ''
        for lSection in self:
            if lSection.title == '':
                lString += '\t<unnamed>' + ' (' + lSection.id + '):  '
            else:
                lString += '\t' + lSection.title
                lString += ' (' + lSection.id + '):  '
            lString += lSection.source() + ' -> ' + lSection.target + '\n'
        return lString

    def toXml(self, aDocument):
        lSetNode = aDocument.createElementNS(EMPTY_NAMESPACE, 'sections')
        for lSection in self:
            lSectionNode = lSection.toXml(aDocument)
            lSetNode.appendChild(lSectionNode)
        return lSetNode



class XsltSet(dict):
    """A dictionnary to manage XSL stylesheets.

    A site needs some stylesheets so as to get its articles and generated
    indexes transformed. Achille gives default stylesheets for each of these
    kinds of documents, but they all can be changed. To achieve this, this
    dictionnary associates each kind of document to its stylesheet.
    """
    def __init__(self, aNode = None):
        dict.__init__(self)
        lPath = os.path.join(site.CONFIGURATION_PATH, 'xsl')
        self.__setitem__(XSLT_TYPE_INDEX, os.path.join(lPath,
                                                       'indexes',
                                                       'index.xsl'))
        self.__setitem__(XSLT_TYPE_ARTICLE, os.path.join(lPath,
                                                         'docbook',
                                                         'page.xsl'))
        self.__setitem__(XSLT_TYPE_FILES, os.path.join(lPath,
                                                       'files',
                                                       'files.xsl'))
        if aNode != None:
            self.__parseNode(aNode)

    def __str__(self):
        lString = ''
        for lKey in self.keys():
            lString += '\t' + lKey + ': ' + self.__getitem__(lKey) + '\n'
        return lString

    def __parseNode(self, aNode):
        lRoot = aNode.getAttributeNS(None, XSLT_ATTR_ROOT)
        for lNode in aNode.childNodes:
            if lNode.nodeName == 'xslt-sheet':
                lType = lNode.getAttributeNS(None, XSLT_ATTR_TYPE)
                lSource = Source(lRoot, lNode.firstChild.nodeValue)
                self.__setitem__(lType, lSource())
    
    def toXml(self, aDocument):
        lSetNode = aDocument.createElementNS(EMPTY_NAMESPACE, 'xslt-set')
        for lKey in self.keys():
            lSheetNode = aDocument.createElementNS(EMPTY_NAMESPACE, 'xslt-sheet')
            lSheetNode.setAttributeNS(EMPTY_NAMESPACE, 'type', lKey)
            lSheetNode.appendChild(
                aDocument.createTextNode(self.__getitem__(lKey)))
            lSetNode.appendChild(lSheetNode)
        return lSetNode

    

class CssSet(dict):
    """A dictionnary to manage stylesheets.

    A site should have a CSS stylesheet. But it can also have alternate sheets,
    so as to be viewable on different media for example..
    
    This dictionnary holds two keys; CSS_TYPE_NORMAL and CSS_TYPE_ALTERNATE.
    For the former key, it maintains a tuple with the name and the filename of
    the main CSS stylesheet. The second key holds a list of these tuples.
    """
    
    def __init__(self, aNodes = None):
        dict.__init__(self)
        self.__setitem__(CSS_TYPE_NORMAL, None)
        self.__setitem__(CSS_TYPE_ALTERNATE, [])

        if aNodes != None:
            for lNode in aNodes:
                lName = lNode.getAttributeNS(None, CSS_ATTR_NAME)
                #if lName == '':
                #    lName = 'unnamed'
                lType = lNode.getAttributeNS(None, CSS_ATTR_TYPE)
                lFilename = lNode.firstChild.nodeValue
                if lType == CSS_TYPE_ALTERNATE:
                    self.__getitem__(CSS_TYPE_ALTERNATE).append(
                        (lName, lFilename))
                elif lType == CSS_TYPE_NORMAL or lType == '':
                    self.__setitem__(CSS_TYPE_NORMAL, (lName, lFilename))
                else:
                    raise AttributeError('invalid css type %s' % lType)

    def __str__(self):
        lString = ''
        for lKey in self.keys():
            lString += '\t' + lKey + ': ' + str(self.__getitem__(lKey)) + '\n'
        return lString

    def addSheet(self, aName, aFile):
        self.__getitem__(CSS_TYPE_ALTERNATE).append((aName, aFile))

    def makeSheetPrincipal(self, aName):
        print 'TODO: CSS.makeSheetPrincipal'
        
    def toXml(self, aDocument):
        lSetNode = aDocument.createElementNS(EMPTY_NAMESPACE, 'css-set')
        try:
            lName, lFile = self.__getitem__(CSS_TYPE_NORMAL)
            lSheetNode = aDocument.createElementNS(
                EMPTY_NAMESPACE, 'css-sheet')
            lSheetNode.setAttributeNS(EMPTY_NAMESPACE, 'name', lName)
            lSheetNode.appendChild(aDocument.createTextNode(lFile))
            lSetNode.appendChild(lSheetNode)

            for lName, lFile in self.__getitem__(CSS_TYPE_ALTERNATE):
                lSheetNode = aDocument.createElementNS(
                    EMPTY_NAMESPACE, 'css-sheet')
                lSheetNode.setAttributeNS(EMPTY_NAMESPACE, 'name', lName)
                lSheetNode.setAttributeNS(
                    EMPTY_NAMESPACE, 'type', CSS_TYPE_ALTERNATE)
                lSheetNode.appendChild(aDocument.createTextNode(lFile))
                lSetNode.appendChild(lSheetNode)

        except KeyError:
            pass

        return lSetNode



class Descriptor(object):
    """The site descriptor abstraction.
    """

    def __init__(self, aFilename, aNode = None):
        self.__dict__['filename'] = aFilename
        self.__dict__['title'] = self._getSiteTitle(aNode)
        self.__dict__['sections'] = self._buildSectionsSet(aNode)
        self.__dict__['cssSet'] = self._buildCssSet(aNode)
        self.__dict__['xsltSet'] = self._buildXsltSet(aNode)
        if aNode != None:
            lTarget = aNode.getAttributeNS(None, 'target')
        else:
            lTarget = 'html'
        self.__dict__['target'] = lTarget
        

    def getSection(self, aId):
        for lSection in self.sections:
            if lSection.id == aId:
                return lSection

    def addSection(self, aSection = None):
        lSection = aSection
        if lSection != None:
            self.__dict__['sections'].append()
        else:
            lSection = Section(self, None, None)
            self.__dict__['sections'].append(lSection)
        return lSection
    
        
    def setTitle(self, aTitle):
        self.__dict__['title'] = aTitle


    def setTarget(self, aTarget):
        self.__dict__['target'] = aTarget


    def toXml(self):
        lDocument = implementation.createDocument(None, u'site', None)
        lDocument.documentElement.setAttributeNS(
            EMPTY_NAMESPACE, 'target', self.target)

        lTitleNode = lDocument.createElementNS(EMPTY_NAMESPACE, 'title')
        lTitleNode.appendChild(lDocument.createTextNode(self.title))
        lDocument.documentElement.appendChild(lTitleNode)

        lDocument.documentElement.appendChild(
            self.xsltSet.toXml(lDocument))

        lDocument.documentElement.appendChild(
            self.cssSet.toXml(lDocument))

        lDocument.documentElement.appendChild(
            self.sections.toXml(lDocument))
        
        return lDocument

    
    def _getSiteTitle(self, aNode):
        if aNode == None:
            return 'Untitled descriptor'
        
        lNodes = XPath.Evaluate(SITE_TITLE_XPATH, aNode)
        if lNodes != []:
            return lNodes[0].firstChild.nodeValue


    def _buildXsltSet(self, aNode):
        if aNode == None:
            return XsltSet()
        
        lNodes = XPath.Evaluate(XSLT_SHEETS_XPATH, aNode)
        if lNodes != []:
            return XsltSet(lNodes[0])
    
    def _buildCssSet(self, aNode):
        if aNode == None:
            return CssSet()

        lNodes = XPath.Evaluate(CSS_SHEETS_XPATH, aNode)
        return CssSet(lNodes)

    def _buildSectionsSet(self, aNode):
        lSections = SectionsSet()
        if aNode != None:
            lNodes = XPath.Evaluate(SECTIONS_XPATH, aNode)
            for lNode in lNodes:
                lSections.extend(SectionsSet(self, lNode))
        return lSections

    def __setattr__(self, aName, aValue):
        raise AttributeError('readonly %s' % aName)

    def __str__(self):
        lString = ''
        lString += 'Descriptor: ' + self.filename + '\n'
        lString += 'Target: ' + self.target + '\n'
        lString += 'Title: ' + self.title + '\n'
        lString += 'CSS sheets:\n' + str(self.cssSet) + '\n'
        lString += 'XSLT sheets:\n' + str(self.xsltSet) + '\n'
        lString += 'Sections:\n' + str(self.sections) + '\n'
        return lString


class Parser(object):
    """The reader of the site descriptor files.

    The Parser object opens the given configuration file,
    builds structures, and closes the file. It then holds structures
    for every aspect of the configuration : an Xslt files Set, a CSS
    files set, and a sections list."""

    def __init__(self):
        self._reader = Domlette.NonvalidatingReader

    def parse(self, aFilename):
        lDocument = self._reader.parseUri(Uri.OsPathToUri(aFilename))
        lDescriptor = Descriptor(aFilename, lDocument.documentElement)
        return lDescriptor


class Writer(object):
    """An object for saving descriptor files.
    """

    def __init__(self):
        self._writer = None

    def write(self, aDescriptor, aStream):
        PrettyPrint(aDescriptor.toXml(), aStream, 'ISO-8859-15')

