# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, sys, thread
import gtk, gtk.glade

import Ft, Ft.Lib

GLADE_FILE = '/usr/local/share/achille/gui/gtk/gtk-achille.glade'

from output import OutputController, AchilleAlreadyRunningException
from tree import TreeController
from notebook import NotebookManager

from achille import config, Worker, GenerationOptions
from achille.lib import files

import achille.lib.log as log

        
class GenerationOptionsController(object):
    GENERATION_DIALOG = 'generationDialog'

    INDEXES_BUTTON = 'indexesButton'
    DOCBOOK_BUTTON = 'docbookButton'
    FILES_BUTTON = 'filesButton'
    OVERWRITE_BUTTON = 'overwriteButton'
    VALIDATE_BUTTON = 'validateButton'
    ASK_BUTTON = 'askButton'
    
    def __init__(self, aOptions):
        self._options = aOptions
        self._design = gtk.glade.XML(GLADE_FILE, self.GENERATION_DIALOG)
        self._dialog = self._design.get_widget(self.GENERATION_DIALOG)
        lDict = {
            'on_indexesButton_toggled': (self.onIndexes),
            'on_docbookButton_toggled': (self.onDocbook),
            'on_filesButton_toggled': (self.onFiles),
            'on_overwriteButton_toggled': (self.onOverwrite),
            'on_validateButton_toggled': (self.onValidate),
            'on_askButton_toggled': (self.onAsk),
            'on_cancelbutton_clicked': (self.onOk),
            'on_okbutton_clicked': (self.onCancel)
            }
        self._design.signal_autoconnect(lDict)
        self._initialize()


    def _initialize(self):
        self._design.get_widget(self.INDEXES_BUTTON).set_active(
            self._options[GenerationOptions.OPTION_INDEXES])
        self._design.get_widget(self.DOCBOOK_BUTTON).set_active(
            self._options[GenerationOptions.OPTION_DOCBOOK])
        self._design.get_widget(self.FILES_BUTTON).set_active(
            self._options[GenerationOptions.OPTION_FILES])
        self._design.get_widget(self.OVERWRITE_BUTTON).set_active(
            self._options[GenerationOptions.OPTION_OVERWRITE])
        self._design.get_widget(self.VALIDATE_BUTTON).set_active(
            self._options[GenerationOptions.OPTION_VALIDATE])
        self._design.get_widget(self.ASK_BUTTON).set_active(
            self._options[GenerationOptions.OPTION_ASK])

        self._design.get_widget(self.OVERWRITE_BUTTON).set_sensitive(False)


    def run(self):
        return self._dialog.run()

        
    def onIndexes(self, aButton):
        self._options[GenerationOptions.OPTION_INDEXES] = aButton.get_active()


    def onDocbook(self, aButton):
        self._options[GenerationOptions.OPTION_DOCBOOK] = aButton.get_active()


    def onFiles(self, aButton):
        self._options[GenerationOptions.OPTION_FILES] = aButton.get_active()


    def onOverwrite(self, aButton):
        self._options[GenerationOptions.OPTION_OVERWRITE] = aButton.get_active()


    def onValidate(self, aButton):
        self._options[GenerationOptions.OPTION_VALIDATE] = aButton.get_active()


    def onAsk(self, aButton):
        self._options[GenerationOptions.OPTION_ASK] = aButton.get_active()


    def onOk(self, aButton):
        self._dialog.hide()


    def onCancel(self, aButton):
        self._dialog.hide()


	
class Application(object):
    ROOT_WINDOW = 'mainWindow'
    OPEN_DIALOG = 'openDialog'
    ABOUT_DIALOG = 'aboutDialog'
    ERROR_DIALOG = 'errorDialog'
    STATUS_CONTEXT = 0
    
    def __init__(self, aDescriptorFile):
        self._design = gtk.glade.XML(GLADE_FILE, self.ROOT_WINDOW)

        # Create controlers
        self._notebook = NotebookManager(
            self._design.get_widget('notebook'))
        self._output = OutputController(
            self, self._design.get_widget('outputView'))
        self._tree = TreeController(
            self, self._design.get_widget('treeView'))
        self._statusbar = self._design.get_widget('statusbar')

        # Plug callbacks
        lDict = {
            'on_new_activate': (self.onMenuNew),
            'on_open_activate': (self.onMenuOpen),
            'on_close_activate': (self.onMenuClose),
            'on_save_activate': (self.onMenuSave),
            'on_saveas_activate': (self.onMenuSaveAs),
            'on_generate_activate': (self.onMenuGenerate, True),
            'on_quit_activate': (gtk.mainquit),
            'on_about_activate': (self.onMenuAbout),
            'on_generateButton_clicked': (self.onMenuGenerate, False),
            'on_treeView_row_activated': (self._tree.onSelection),
            'on_treeView_button_press_event': (self._tree.onButtonPressEvent),
            'on_mainWindow_delete_event': (gtk.mainquit)
            }
        self._design.signal_autoconnect(lDict)


        # Generation options
        self._generationOptions = GenerationOptions()
        self._descriptor = None
        if aDescriptorFile != None and aDescriptorFile != '':
            self.parseDescriptor(aDescriptorFile)
            
        self._worker = None
        #FIXME
        log.createWithSocket('achille', aLevel = log.DEBUG)


    def getDescriptor(self):
        return self._descriptor
    
        
    def onMenuNew(self, aWidget):
        self._descriptor = config.Descriptor('d.xml', None)
        self._tree.buildFrom(self._descriptor)
        self.openDescriptor()


    def onMenuOpen(self, aWidget):
        if self._worker != None:
            self.showError(
                'Error', 'Please wait the current generation has finished.')
            return gtk.FALSE

        lDesign = gtk.glade.XML(GLADE_FILE, self.OPEN_DIALOG)
        lDialog = lDesign.get_widget(self.OPEN_DIALOG)
        lDesign.get_widget('okbutton').connect(
            'clicked', lambda lWidget: lDialog.hide())
        lDesign.get_widget('okbutton').connect(
            'clicked', lambda lWidget: self.openNow(lDesign))
        lDesign.get_widget('cancelbutton').connect(
            'clicked', lambda lWidget: lDialog.hide())
        lDialog.run()
        return gtk.FALSE


    def onMenuSave(self, aWidget):
        self.writeDescriptor()


    def onMenuSaveAs(self, aWidget):
        lDialog = gtk.FileSelection('Save a descriptor...')
        lDialog.set_filename('descriptor.xml')
        lDialog.ok_button.connect(
            'clicked',
            lambda lWidget: self.writeDescriptor(lDialog.get_filename()))
        lDialog.ok_button.connect(
            'clicked', lambda lWidget: lDialog.destroy())
        lDialog.cancel_button.connect(
            'clicked', lambda lWidget: lDialog.destroy())
        lDialog.hide_fileop_buttons()
        lDialog.run()
        return gtk.FALSE

        
    def onMenuClose(self, aWidget):
        self._notebook.closeCurrentTab()
        return gtk.FALSE


    def onMenuGenerate(self, aWidget, aFromMenu):
        if self._descriptor == None:
            self.showError('Generation error', 'No opened descriptor.')
            return gtk.FALSE

        if self._worker != None:
            self.showError('Generation error', 'Generation is already running.')
            return gtk.FALSE

        # Eventually ask for options and open thread
        if aFromMenu or self._generationOptions[GenerationOptions.OPTION_ASK]:
            lDialog = GenerationOptionsController(self._generationOptions)
            if lDialog.run() != gtk.RESPONSE_OK:
                return gtk.FALSE

        self._worker = thread.start_new_thread(
            Worker.generate, (self._descriptor, self._generationOptions))
        return gtk.FALSE

        
    def onMenuAbout(self, aWidget):
        lXmlDesign = gtk.glade.XML(GLADE_FILE, self.ABOUT_DIALOG)
        lDialog = lXmlDesign.get_widget('aboutDialog')
        lXmlDesign.get_widget('okbutton').connect(
            'clicked', lambda lWidget: lDialog.hide())
        lDialog.run()
        return gtk.FALSE
    
        
    def parseDescriptor(self, aFilename):
        try:
            self._descriptor = config.Parser().parse(aFilename)
            self._tree.buildFrom(self._descriptor)
            self._statusbar.push(self.STATUS_CONTEXT,
                                 'Descriptor: ' + aFilename)
        except Ft.Lib.UriException:
            self.showError('Descriptor error',
                           'No file named %s.' % (aFilename))
            return gtk.FALSE
            

    def writeDescriptor(self, aFilename = '', aStream = sys.stdout):
        if len(aFilename):
            lFile = open(aFilename, 'w')
        else:
            lFile = aStream
        config.Writer().write(self._descriptor, lFile)
        self._statusbar.push(
            self.STATUS_CONTEXT, 'Descriptor saved in ' + lFile.name)


    def openNow(self, aDesign):
        if aDesign.get_widget('descriptorRadiobutton').get_active(): 
            lDialog = gtk.FileSelection('Open a descriptor...')
            lDialog.set_filename('descriptor.xml')
            lDialog.ok_button.connect(
                'clicked',
                lambda lWidget: self.parseDescriptor(lDialog.get_filename()))
        elif aDesign.get_widget('fileRadiobutton').get_active(): 
            lDialog = gtk.FileSelection('Open a file...')
            lDialog.ok_button.connect(
                'clicked',
                lambda lWidget: self.openTabFor(lDialog.get_filename()))
        else:
            raise RuntimeError('No valid type to open')
        
        lDialog.ok_button.connect(
            'clicked', lambda lWidget: lDialog.hide())
        lDialog.cancel_button.connect(
            'clicked', lambda lWidget: lDialog.hide())
        lDialog.hide_fileop_buttons()
        lDialog.run()

        
    def openTabFor(self, aLink):
        self._notebook.open(aLink)
		

    def openSectionFor(self, aId):
        self._notebook.open(self._descriptor.getSection(aId))

        
    def openDescriptor(self):
        self._notebook.open(self._descriptor)

            

    def showError(self, aTitle, aDetail):
        lXmlDesign = gtk.glade.XML(GLADE_FILE, self.ERROR_DIALOG)
        lDialog = lXmlDesign.get_widget(self.ERROR_DIALOG)
        lXmlDesign.get_widget('titleLabel').set_text(
            '<span size="larger">' + aTitle + '</span>')
        lXmlDesign.get_widget('detailLabel').set_text(aDetail)
        lXmlDesign.get_widget('closeButton').connect(
            'clicked', lambda lWidget: lDialog.hide())
        lDialog.run()
