# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, logging, sys, re, socket, thread
import pickle
import struct

import gtk, gobject, pango


class AchilleAlreadyRunningException(object):
    pass


class OutputReader(object):
    def __init__(self, aApp, aView):
        self._application = aApp
        self._view = aView
        self._buffer = aView.get_buffer()
        #        self._linkExpression = re.compile(
        #            r'(?P<text>[\s\w]*)\'(?P<link>[\w\s\/\\\.]*)\'')
        self._linkExpression = re.compile(
            r'(?P<text_before>.*)\s(?P<link>(\w*\/\w*)+\.*\w*)')


    def onParseableData(self, aFileDescriptor, aCondition):
        #print '** Reader running...'
        lPacketSize = struct.unpack('>L', os.read(aFileDescriptor, 4))[0]
        lPacket = os.read(aFileDescriptor, lPacketSize)
        lRecord = logging.makeLogRecord(pickle.loads(lPacket))
        self._insert(lRecord)
        #print '** Reader stopping.'
        return gtk.TRUE


    def _insert(self, aRecord):
        self._buffer.insert_with_tags_by_name(
            self._buffer.get_end_iter(), aRecord.filename, 'filename')
        self._tabulate(20 - len(aRecord.filename))
        
        self._buffer.insert_with_tags_by_name(
            self._buffer.get_end_iter(), aRecord.levelname, aRecord.levelname)
        self._tabulate(10 - len(aRecord.levelname))

        try:
            lMessage = aRecord.msg.strip()
            lMatch = self._linkExpression.search(lMessage)
            self._buffer.insert_with_tags_by_name(
                self._buffer.get_end_iter(),
                lMatch.group('text_before') + ' ', aRecord.levelname)
            self._insertLink(lMatch.group('link'), aRecord.levelname)
                             
        except AttributeError:
            self._buffer.insert_with_tags_by_name(
                self._buffer.get_end_iter(), lMessage, aRecord.levelname) 
           
            pass
        self._buffer.insert(self._buffer.get_end_iter(), '\n')


    def _insertLink(self, aLink, aTagname):
        #lRootDir = os.path.split(self._application.getDescriptor().filename)[0]
        #if os.path.isfile(os.path.join(lRootDir, aLink)):
        if os.path.isfile(aLink):
            self._buffer.insert_with_tags_by_name(
                self._buffer.get_end_iter(), aLink, 'link')
        else:
            self._buffer.insert_with_tags_by_name(
                self._buffer.get_end_iter(), aLink, aTagname)
            

    def _tabulate(self, aLength):
        self._buffer.insert_with_tags_by_name(
            self._buffer.get_end_iter(), aLength * ' ', 'text')

        

class OutputController(object):
    def __init__(self, aApp, aView):
        self._application = aApp
        self._view = aView
        self._buffer = aView.get_buffer()
        self._colormap = aView.get_colormap()
        self._reader = None
        self._initTags()
        self._reader = OutputReader(self._application, self._view)

        thread.start_new_thread(self._runLogServer, ())


    def _runLogServer(self):
        HOST, PORT = '', 5000
        lSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        lSocket.bind((HOST, PORT))
        lSocket.listen(1)
        while 1:
            lConnection, lAddress = lSocket.accept()
            gobject.io_add_watch(
                lConnection.fileno(), gobject.IO_IN, self._reader.onParseableData)


    def _initTags(self):
        # Default output
        lTag = self._buffer.create_tag('text', family = 'Monospace')

        # Link
        lColor = self._colormap.alloc_color(0, 0, 0xaaaa)
        lTag = self._buffer.create_tag('link',
                                       foreground_gdk = lColor,
                                       family = 'Monospace',
                                       underline = pango.UNDERLINE_SINGLE)
        lTag.connect('event', self.onLinkClicked)

        # DEBUG level and message
        lColor = self._colormap.alloc_color(0xaaaa, 0x0000, 0xaaaa)
        lTag = self._buffer.create_tag('DEBUG',
                                       foreground_gdk = lColor,
                                       family = 'Monospace')

        # INFO level and message
        lColor = self._colormap.alloc_color(0x0000, 0xffff, 0x0000)
        lTag = self._buffer.create_tag('INFO',
                                       foreground_gdk = lColor,
                                       family = 'Monospace')

        # WARNING level and message
        lColor = self._colormap.alloc_color(0xaaaa, 0xaaaa, 0x0000)
        lTag = self._buffer.create_tag('WARNING',
                                       foreground_gdk = lColor,
                                       family = 'Monospace',
                                       weight = pango.WEIGHT_BOLD)

        # ERROR level and message
        lColor = self._colormap.alloc_color(0xaaaa, 0x0000, 0x0000)
        lTag = self._buffer.create_tag('ERROR',
                                       foreground_gdk = lColor,
                                       family = 'Monospace',
                                       weight = pango.WEIGHT_BOLD)

        # Filename
        lColor = self._colormap.alloc_color(0xaaaa, 0xaaaa, 0xaaaa)
        lTag = self._buffer.create_tag('filename',
                                       foreground_gdk = lColor,
                                       family = 'Monospace')


    def onLinkClicked(self, aTag, aWidget, aEvent, aIter):
        if aEvent.type != gtk.gdk.BUTTON_PRESS:
            return gtk.FALSE

        beginIter = aIter.copy()
        while not beginIter.begins_tag(aTag):
            beginIter.backward_char()

        while not aIter.ends_tag(aTag):
            aIter.forward_char()

        self._application.openTabFor(
            beginIter.get_slice(aIter))
        
        return gtk.FALSE

