# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os

from Ft.Lib import Uri
from Ft.Xml import XPath
from Ft.Xml import Domlette
from Ft.Xml import EMPTY_NAMESPACE

import achille.lib.log as log
import achille.lib.date as date

# FIXME
from indexbuilder import *
from achille.config.constants import *
from achille.lib.files import FileSystemInterface


ARTICLE_TITLE_XPATH    = '/article/title'
ARTICLE_DATE_XPATH     = '/article/artheader/date'
ARTICLE_MEDIA_XPATH     = '/article/artheader/mediaobject/imageobject/imagedata'
ARTICLE_ABSTRACT_XPATH = '/article/artheader/abstract'

DC_NAMESPACE = 'http://purl.org/dc/elements/1.1/'
IMAGE_NAMESPACE = 'http://purl.org/rss/1.0/modules/image/'
RDF_NAMESPACE = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'


class SummaryIndexBuilder(IndexBuilder):
    def __init__(self, aSection):
        IndexBuilder.__init__(self, aSection)
        self._nbArticles = 0
        
    def run(self):
        log.get(LOG_WORKER).info(
            ('\tBuilding summary in %s' % self._section.source))
        for lDir in self._section.index.dirs:
            self.readDirectory(lDir)
        
        self.createIndex(None, None, RDF_NAMESPACE, 'rdf:RDF')
        lChannelNode = self._document.createElementNS(EMPTY_NAMESPACE,
                                                      'channel')
        lTitleNode = self._document.createElementNS(EMPTY_NAMESPACE,
                                                      'title')
        lTitleNode.appendChild(self._document.createTextNode(self._section.title))
        lItemsNode = self._document.createElementNS(EMPTY_NAMESPACE, 'items')
        lSeqNode = self._document.createElementNS(RDF_NAMESPACE, 'rdf:Seq')

        lItemsNode.appendChild(lSeqNode)
        lChannelNode.appendChild(lTitleNode)
        lChannelNode.appendChild(lItemsNode)
        self._document.documentElement.appendChild(lChannelNode)

        # The articles map
        lArticles = {}
        lMaxArticles = self._section.index.max
        for lFile in self._files[FileSystemInterface.TYPE_DTD_DOCBOOK]:
            if lMaxArticles != 0 and len(lArticles.keys()) >= lMaxArticles:
                log.get(LOG_WORKER).info(('\tWill drop %s' % lFile))
                continue
            
            if lFile in self._section.index.ignoredFiles:
                log.get(LOG_WORKER).info(('\tWill ignore %s' % lFile))
                continue

            lKey, lId, lNode = self.createArticleNode(lFile, self._section.index.featuredFile)
            if lArticles.has_key(lKey):
                log.get(LOG_WORKER).warn(('\tDuplicate key: %s' % lKey))
                
            lArticles[lKey] = lId, lNode

        lKeys = lArticles.keys()
        lKeys.sort()
        if self._section.index.sort in [INDEX_SORT_DATE, '']:
            lKeys.reverse()
        for lKey in lKeys:
            lId, lNode = lArticles[lKey]
            lItemNode = self._document.createElementNS(RDF_NAMESPACE,
                                                       'rdf:li')
            lItemNode.setAttributeNS(EMPTY_NAMESPACE, 'resource', lId)
            lSeqNode.appendChild(lItemNode)
            self._document.documentElement.appendChild(lNode)
            
        self.writeIndex(FileSystemInterface.TYPE_SCHEMA_RDF)


    def createArticleNode(self, aFilename, aFeaturedFile):
        # Useful for sorting... see below
        lTitle = ''
        lDate = ''
        
        log.get(LOG_WORKER).debug(('\tReading article: %s' % aFilename))
        lNewNode = self._document.createElementNS(EMPTY_NAMESPACE, 'item')
        lNewNode.setAttributeNS('rdf', 'about', aFilename)

        # Build article XML tree
        lReader = Domlette.NoExtDtdReader
        lDocument = lReader.parseUri(Uri.OsPathToUri(aFilename))

        # Get article title
        lNodes = XPath.Evaluate(ARTICLE_TITLE_XPATH, lDocument.documentElement)
        if lNodes != []:
            lNode = self._document.importNode(lNodes[0], 1)
            lNewNode.appendChild(lNode)
            lTitle = lNode.firstChild.nodeValue

        # Get article link
        lPath = aFilename.replace(self._section.source(), '')
        lPath = lPath.replace(os.sep, '/')
        if lPath[0] == '/':
            lPath = lPath[1:]

        lLinkNode = self._document.createElementNS(EMPTY_NAMESPACE, 'link')
        lLinkNode.appendChild(self._document.createTextNode(lPath))
        lNewNode.appendChild(lLinkNode)

        # Get image
        lNodes = XPath.Evaluate(ARTICLE_MEDIA_XPATH, lDocument.documentElement)
        if lNodes != []:
            lUrl = lNodes[0].attributes[(None, 'fileref')].value
            lMediaNode = self._document.createElementNS(IMAGE_NAMESPACE,
                                                        'image:item')
            lMediaNode.setAttributeNS(RDF_NAMESPACE, 'rdf:about', lUrl)
            lNewNode.appendChild(lMediaNode)

        # Get article description
        # [TODO] We should pick up to 500 characters to put here...
        # [TODO] If no description is present, we should use the first
        # n characters of the article...
        lNodes = XPath.Evaluate(
            ARTICLE_ABSTRACT_XPATH, lDocument.documentElement)
        if lNodes != []:
            lInfoNode = self._document.createElementNS(EMPTY_NAMESPACE,
                                                       'description')
            for lChildNode in lNodes[0].childNodes:
                lNode = self._document.importNode(lChildNode, 1)
                lInfoNode.appendChild(lNode)
            lNewNode.appendChild(lInfoNode)

        # Get date
        lNodes = XPath.Evaluate(ARTICLE_DATE_XPATH, lDocument.documentElement)
        if lNodes != []:
            lNode = self._document.importNode(lNodes[0], 1)
            lDate = lNode.firstChild.nodeValue
        else:
            lDate = date.getFileModificationDate(aFilename)

        lDateNode = self._document.createElementNS(DC_NAMESPACE, 'dc:date')
        lDateNode.appendChild(self._document.createTextNode(lDate))
        lNewNode.appendChild(lDateNode)

        # Is article featured ?
        if aFilename == aFeaturedFile:
            lTypeNode = self._document.createElementNS(DC_NAMESPACE, 'dc:type')
            lTypeNode.appendChild(self._document.createTextNode('featured'))
            lNewNode.appendChild(lTypeNode)
        
        self._nbArticles += 1

        # Choose the key to return
        lSplitDate = lDate.split('/')
        lKey = ''
        try:
            lKey = lSplitDate[2] + lSplitDate[1] + lSplitDate[0]
        except IndexError:
            log.get(LOG_WORKER).error('\t** Error while parsing date')
        
        lKey += str(self._nbArticles)
        if self._section.index.sort == INDEX_SORT_TITLE:
            lKey = lTitle
        elif self._section.index.sort == INDEX_SORT_FILE:
            lKey = aFilename

        return lKey, aFilename, lNewNode
