# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os, shutil, time

import Ft.Xml
import Ft.Lib

import achille.lib.log as log
from achille.config.constants import LOG_LIB

IMAGES_EXT = ['jpg', 'png']

IMAGES_IDX = 'images'
UNKNOWN_IDX = 'unknown'

# [TODO] give abstraction here : make no use of sections.

class CachedDirectory(object):
    def __init__(self, aDirectory):
        self.__name = aDirectory
        self.__synchronize()
        

    def getMap(self):
        return self.__cache

    
    def getSchemaFor(self, aDocument):
        try:
            lNode = aDocument.childNodes[0]
            try:
                lNamespace = u'xmlns:' + lNode.nodeName.split(':')[0]
            except ValueError:
                lNamespace = u'xmlns'
                
            for lKey in lNode.attributes:
                if lNode.attributes[lKey].name == lNamespace:
                    return lNode.attributes[lKey].value
            return ""
        except AttributeError, e:
            # No DOCTYPE element
            # print '??', __name__, '\t\tNo Schema found'
            return ""


    def getDocumentTypeFor(self, aFile):
        try:
            lReader = Ft.Xml.Domlette.NoExtDtdReader
            lDocument = lReader.parseUri(Ft.Lib.Uri.OsPathToUri(aFile))
        except Ft.Xml.ReaderException, e:
            # Not an XML tree
            log.get(LOG_LIB).error(
                ('\t...\t** Bad format for %s: %s' % (aFile, e)))
            return ""
        except IOError, e:
            # Probably a directory
            log.get(LOG_LIB).warn(
                ('\t...\t** Bad file %s: %s' % (aFile, e)))
            return ""
        
        lType = self.getDTDFor(lDocument)
        if lType != "":
            return lType
        lType = self.getSchemaFor(lDocument)
        return lType


    def getDTDFor(self, aDocument):
        try:
	    return os.path.basename(aDocument.systemId)
        except AttributeError, e:
            # No DOCTYPE element... keep quiet with this
            # print '??', __name__, '\t\tNo DTD found'
            return ""


    def addFile(self, aType, aFile):
        if not aType in self.__cache.keys():
            self.__cache[aType] = []
        self.__cache[aType].append(aFile)


    def __synchronize(self):
        """Build this object's cache from filesystem.
        """
        log.get(LOG_LIB).debug(('\t...\tFetching content of %s' % self.__name))
        self.__cache = {}
        try:
            lFiles = os.listdir(self.__name)
            for lFile in lFiles:
                # Forget hidden files
                if lFile.startswith('.'):
                    continue

                # Sort found files
                lExtension = os.path.splitext(lFile)[1].lower()
		lFile = os.path.join(self.__name, lFile)
		if lExtension == '.xml':
                    lType = self.getDocumentTypeFor(lFile)
		    self.addFile(lType, lFile)
                elif lExtension in IMAGES_EXT:
                    self.addFile(IMAGES_IDX, lFile)
                else:
                    self.addFile(UNKNOWN_IDX, lFile)
        except OSError, e:
            log.get(LOG_LIB).error(('\t...\t** %s' % e))


            
class FileSystemInterface(object):
    TYPE_DTD_DOCBOOK = u'docbookx.dtd'
    TYPE_DTD_FILES_INDEX = u'achille-file-index.dtd'
    TYPE_SCHEMA_RDF = u'http://www.w3.org/1999/02/22-rdf-syntax-ns#'
    
    __instance = None
    
    def __new__(cls, *args, **kwds):
        if cls.__instance == None:
            cls.__instance = object.__new__(cls)
        return cls.__instance

    def __init__(self):
        # Users should use get()
        self.__directories = {}
        __instance = self
        

    def get(cls):
        if cls.__instance == None:
            FileSystemInterface()
        return cls.__instance
    get = classmethod(get)


    def clearIndexIn(self, aDir):
        try:
            os.remove(os.path.join(aDir, 'index.xml'))
        except OSError:
            pass


    def readDirectory(self, aDirectory):
        if not aDirectory in self.__directories.keys():
            self.__directories[aDirectory] = CachedDirectory(aDirectory)
        return self.__directories[aDirectory].getMap()


    def addFileFor(self, aDirectory, aType, aFile):
        if not aDirectory in self.__directories.keys():
            self.readDirectory(aDirectory)
        self.__directories[aDirectory].addFile(aType, aFile)


    def copy(self, aWorkingDir, aSection):
        log.get(LOG_LIB).debug(
            ('\t...\tCopying content of %s' % aSection.source()))
        buildPathFor(aWorkingDir, aSection)
        lFiles = os.listdir(aSection.source())
        for lFile in lFiles:
            if os.path.splitext(lFile)[1][1:] in aSection.copiedFiles or '*' in aSection.copiedFiles:
                lTargetFile = os.path.join(
                    aWorkingDir, aSection.target[1:], lFile)
                shutil.copyfile(os.path.join(aSection.source(), lFile),
                                lTargetFile)


def buildPathFor(aWorkingDir, aSection):
    lDirectory = os.path.join(aWorkingDir, aSection.target[1:])
    if not os.path.isdir(lDirectory):
        os.makedirs(lDirectory)


def buildTargetFile(aPath, aFilename):
    return os.path.join(aPath, os.path.split(aFilename)[1])

   
def removePath(aPath):
    lFiles = os.listdir(aPath)
    for lFilename in lFiles:
        lFile = os.path.join(aPath, lFilename)
        if os.path.isdir(lFile):
            removePath(lFile)
            try:
                os.rmdir(lFile)
            except OSError:
                log.get(LOG_LIB).error(('\t...\t** No directory %s ' % lFile))
        else:
            log.get(LOG_LIB).warn(('\t...\tDeleting %s' % lFile))
            os.unlink(lFile)
    os.rmdir(aPath)
    

def getPathToRoot(aPath):
    lSectionPath = os.path.join(aPath)
    lRootPath = ''
    for lDir in aPath.split(os.sep):
        if lDir != '':
            lRootPath += '../'
    if lRootPath == '':
        lRootPath = './'
    return lRootPath


def getFileInfo(aFilename):
    """Reads a file information
    Returns the 2-uple : size, date
    """
    lInfo = os.stat(aFilename)
    lTimeTuple = time.localtime(time.time())
    lDate = time.strftime('%A %d %B %Y %H:%M', lTimeTuple)
    return str(lInfo.st_size), lDate
