#!/bin/env python

# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""The main Achille script.

Achille is a set of modules aimed at managing a simple website. Currently,
Achille can transform DocBook XML articles and create missing pages like
indexes for file lists, site sections. It is foreseen to add support to
manage photo galleries, XBEL bookmarks and other format that Docbook.

This script allows easy use of Achille tools by making sanity checks and
sequencing operations. It is also possible to launch a GTK2 graphical
interface to Achille, but this feature is experimental.
"""

import getopt, os, sys

try:
    from Ft.Lib import UriException
except Exception, e:
    print '4Suite is not correctly installed:', e
    sys.exit(1)
    
try:
    from achille import config, Worker, GenerationOptions
    from achille.lib import log
except Exception, e:
    print 'Achille is not correctly installed:', e
    sys.exit(1)
    


class Achille(object):
    """The Achille facility class.

    This class is the Achille-user interface. It allows the user to
    invoke Achille in a simple way, provides a way to tweak some options,
    makes sanity checks and prepare everything for the work to be done. 
    """
    def __init__(self):
        """Achille constructor.
        """
        self._descriptor = ''
        self._command = ''
        self._gui = False
        self._options = GenerationOptions()
        self._logLevel = log.INFO

        self.readCommandLine(sys.argv[1:])

    
    def printUsage(self):
        """Prints a short notice about Achille usage.
        """
        print config.constants.NOTICE
        print 'Usage:'
        print '\trun_achille.py [options] -d descriptor command'
        print '\trun_achille.py [options] [-d descriptor] -g | --gui'


    def printGplConditions(self):
        """Prints a word about the GPL license.
        """
        print config.constants.GPL

        
    def printGplWarranty(self):
        """Prints the warranty section of the GPL license.
        """
        print config.constants.WARRANTY


    def process(self):
        """Starts the demanded task.

        The script shall be used to invoke a simple command (like build
        or check) along with a descriptor name, or to launch the graphical
        user interface.
        """
        log.createWithStream(config.constants.LOG_ROOT, self._logLevel)

        # First change to descriptor directory if descriptor was given
        if self._descriptor != '':
            lPath, self._descriptor = os.path.split(self._descriptor)
            if lPath != '':
                os.chdir(lPath)
                log.get(config.constants.LOG_ROOT).debug('Moving to: ' + lPath)

        # Then start the GUI if requested, or execute the given command
        if self._gui == True:
            log.get(config.constants.LOG_ROOT).debug('Starting GUI')
            self.runGUI()
        else:
            log.get(config.constants.LOG_ROOT).debug('Processing command')
            self.executeCommand()


    def executeCommand(self):
        """Execute the given command.

        This script's role ends here. All data is furnished to Achille
        classes so as to execute the requested order.
        """
        if self._descriptor == '':
            self.printUsage()
            return

        lConf = None
        try:
            lConf = config.Parser().parse(self._descriptor)
        except UriException:
            log.get(config.constants.LOG_ROOT).error(
                'No file named %s.' % (self._descriptor))
            return
        
        if self._command == 'build':
            Worker.generate(lConf, self._options)

        elif self._command == 'clean':
            Worker.clean(lConf)

        elif self._command == 'check':
            Worker.check(lConf)
                
        else:
            self.printUsage()


    def runGUI(self):
        """Execute the GTK2 graphical user interface.

        This script's role ends here. The user interface is started
        if PyGTK was correctly installed. If not, the program stops there.
        """
        try:
            import pygtk
            pygtk.require('2.0')
        except Exception, e:
            print 'No PyGTK 2.0:', e
            sys.exit(1)
        try:    
            import gtk
            import gtk.glade
        except Exception, e:
            print 'No GTK:', e
            sys.exit(1)    
            
        from achille.gui.gtk import Application

        gtk.threads_init()
        Application(self._descriptor)
        gtk.mainloop()

        
    def readCommandLine(self, aLine):
        """
        """
        # Command line options
        OPT_LIST = 'hd:cwvg'
        LONG_OPT_LIST = [
            'help',
            'validate',
            'ignore-indexes',
            'no-transform',
            'descriptor=',
            'conditions',
            'warranty',
            'verbose',
            'gui']
        DESCRIPTION_LIST = [
            'Show this message',
            'Validate the documents when generating XHTML pages',
            'Do not generate indexes',
            'Do not transform documents to XHTML pages',
            'Specify the descriptor to use',
            'Show the usage conditions',
            'Show the warranty',
            'Increase verbosity',
            'Use the graphical interface']
        
        try:
            lOptlist, lArgs = getopt.getopt(aLine, OPT_LIST, LONG_OPT_LIST)
        except getopt.GetoptError:
            self.printUsage()
            sys.exit()

        if len(lArgs) > 0:
            self._command = lArgs[0]
            
        for lOptName, lOptArg in lOptlist:
            if lOptName in ('-h', '--help'):
                self.printUsage()
                print 'Options:'
                for l in range(len(LONG_OPT_LIST)):
                    print '--' + LONG_OPT_LIST[l], '\t', DESCRIPTION_LIST[l]
                sys.exit()
            if lOptName in ('-w', '--warranty'):
                self.printGplWarranty()
                sys.exit()
            if lOptName in ('-c', '--conditions'):
                self.printGplConditions()
                sys.exit()
            if lOptName in ('-v', '--verbose'):
                self._logLevel = log.DEBUG
            if lOptName in ('-g', '--gui'):
                self._gui = True
            if lOptName in ('-d', '--descriptor'):
                self._descriptor = lOptArg
            if lOptName == '--ignore-indexes':
                self._options[GenerationOptions.OPTION_INDEXES] = False
            if lOptName == '--no-transform':
                self._options[GenerationOptions.OPTION_DOCBOOK] = False
            if lOptName == '--validate':
                self._options[GenerationOptions.OPTION_VALIDATE] = True

        
# Main
try:
    Achille().process()
except KeyboardInterrupt:
    log.get(config.constants.LOG_ROOT).error('Stopped by user')
    
except config.ConfigurationError, e:
    log.get(config.constants.LOG_ROOT).error(('Error: %s' % e))
